<?php

/**
 * @file
 * Demonstrates hooks available from the Autolink UI module.
 */

/**
 * Defines types of links that can be defined.
 */
function hook_autolink_link_types() {
  return array(
    'link' => t('Link to path'),
    'search' => t('Link to search results'),
    'taxonomy' => t('Link to taxonomy term'),
  );
}

/**
 * Defines form selection methods for the link add/edit form.
 *
 * This data is used to call callback functions to build AJAX form elements.
 * Elements should define the link type with which the form is related, as
 * well as a callback function for building the form. Optional callback
 * arguments can be specified in the form of an array. The title attribute
 * should be translated with t() and is displayed in the link type dropdown
 * on the link form. Validate functions may also be specified. Callback
 * arguments will be passed to the validate function as well. Form elements
 * that are defined in hook_autolink_form_elements() must specify a link
 * type that has been defined in hook_autolink_link_types().
 */
function hook_autolink_form_elements() {
  $elements['link_url'] = array(
    'type' => 'link',
    'title' => t('Enter a URL'),
    'callback' => 'autolink_ui_link_method',
    'validate' => 'autolink_ui_link_method_validate',
  );
  $elements['node_title'] = array(
    'type' => 'node',
    'title' => t('Select a node'),
    'callback' => 'autolink_ui_node_method',
    'callback arguments' => array('title'),
    'validate' => 'autolink_ui_node_method_validate',
  );
  return $elements;
}

/**
 * Modifies link data before it is inserted into the database.
 *
 * hook_autolink_METHOD_alter allows other modules to alter the
 * form data of a link based on the selection method defined by the external
 * module. Providing this  unique hook allows different types of links
 * and associated paths to be developed, opening possibilities for linking
 * to taxonomy, search, user, and other paths. Selection methods should be
 * defined in a value field named 'operation' on the link add form. Modules
 * that define selection methods must set the link_type in this function
 * before the link is saved.
 *
 * @param &$link
 *   An array containing the link data entered in an administrative form
 *   before save. The hook passes this form data on to other modules so
 *   they may use form data to determine the correct path or query if
 *   necessary. Path should be stored in $link['link_path']. Link paths
 *   are not built on the form so they must be build in a hook if the
 *   module defines a new link type. Paths can be either externam urls
 *   or internal Drupal paths, but Drupal paths that are aliases should
 *   be converted to normally paths before being stored in the database.
 *   Optional queries should be stored in $link['query']. This data will
 *   be used to process link paths when links are generated.
 * @param $method
 *   This parameter is a string representing the form selection method.
 *   Selection methods are defined in hook_autolink_form_elements(),
 *   and may be used to determine how to build paths based on form data.
 *   Modules may call autolink_link_save() with only a $link['method']
 *   attribute defined, and Autolink will correctly set the link type
 *   and call hook_autolink_LINK_TYPE_alter() based on that data.
 *
 * @return
 *   The new form data after it has been altered by external modules.
 *   Link data is passed by reference.
 */
function hook_autolink_LINK_TYPE_alter(&$link, $method) {
  if ($method == 'path') {
    if (drupal_lookup_path('source', $link['argument']) == FALSE) {
      $link['link_path'] = $link['argument'];
    }
    else {
      // Get the normal path if this path is an alias.
      $link['link_path'] = drupal_lookup_path('source', $link['argument']);
    }
    // Set the node ID.
    $args = explode('/', $link['link_path']);
    $link['object_id'] = $args[1];
  }
}

/**
 * Notifies modules that a new link definition was inserted.
 *
 * hook_autolink_insert() is invoked after a link definition has been saved
 * new to the database. This allows external modules to react to the save and
 * update their own database tables, which is done with the autolink_search
 * module.
 *
 * @param $link
 *   An associative array of form data that was saved.
 */
function hook_autolink_link_insert($link) {

}

/**
 * Notifies modules that a link definition was updated.
 *
 * hook_autolink_update() is invoked after a link definition has been updated
 * in the database. This allows external modules to react to the save and
 * update their own database tables, which is done with the autolink_search
 * module.
 *
 * @param $link
 *   An associative array of form data that was saved.
 */
function hook_autolink_link_update($link) {

}

/**
 * Notifies modules that a link definition was deleted.
 *
 * This hook is invoked after the link has already been removed from the database
 * tables. A useful function for this hook is to use it to update another module's
 * own separate but related tables, as the autolink_search module does.
 *
 * @param $lid
 *   The link ID of the link definition that was deleted.
 */
function hook_autolink_link_delete($lid) {

}

/**
 * Notifies modules that a link group has been activated.
 *
 * @param $group
 *   The machine name of the link group that was activated.
 */
function hook_autolink_group_activate($group) {

}

/**
 * Notifies modules that a link group has been disabled.
 *
 * @param $group
 *   The machine name of the link group that was disabled.
 */
function hook_autolink_group_disable($group) {

}

/**
 * Defines bulk operations for use on the link groups overview form.
 */
function hook_autolink_group_operations() {
  return array(
    'unblock' => array(
      'label' => t('Activate the selected link definitions'),
      'callback' => 'autolink_autolink_group_operations_activate',
    ),
    'block' => array(
      'label' => t('Disable the selected link definitions'),
      'callback' => 'autolink_autolink_group_operations_disable',
    ),
  );
}

/**
 * Defines filters for the link groups overview form.
 */
function hook_autolink_filters() {
  return array(
    'status' => array(
      'title' => t('status'),
      'where' => 'a.status = %d',
      'join' => '',
      'options' => array(1 => t('active'), 0 => t('disabled')),
    ),
  );
}

/**
 * Defines bulk operations for use on the link definition overview form.
 */
function hook_autolink_link_operations() {
  return array(
    'delete' => array(
      'label' => t('Activate the selected link definitions'),
      'callback' => 'autolink_autolink_link_operations_delete',
    ),
  );
}
